'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _sourceMapSupport2 = require('source-map-support');

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _firefoxProfile = require('firefox-profile');

var _firefoxProfile2 = _interopRequireDefault(_firefoxProfile);

var _util = require('util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

(0, _sourceMapSupport2.install)();
class FirefoxProfileLauncher {
    onPrepare(config, capabilities) {
        var _this = this;

        return _asyncToGenerator(function* () {
            var _context;

            _this.config = config;
            _this.capabilities = capabilities;

            // Return if no profile options were specified
            if (!_this.config.firefoxProfile) {
                return;
            }

            _this.profile = new _firefoxProfile2.default();

            // Set preferences and proxy
            _this._setPreferences();

            if (!Array.isArray(_this.config.firefoxProfile.extensions)) {
                return _this._buildExtension();
            }

            // Add the extension
            yield (0, _util.promisify)((_context = _this.profile).addExtensions.bind(_context))(_this.config.firefoxProfile.extensions);

            return _this._buildExtension();
        })();
    }

    /**
     * Sets any preferences and proxy
     */
    _setPreferences() {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = Object.entries(this.config.firefoxProfile)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                const _ref = _step.value;

                var _ref2 = _slicedToArray(_ref, 2);

                const preference = _ref2[0];
                const value = _ref2[1];

                if (['extensions', 'proxy'].includes(preference)) {
                    continue;
                }

                this.profile.setPreference(preference, value);
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }

        if (this.config.firefoxProfile.proxy) {
            this.profile.setProxy(this.config.firefoxProfile.proxy);
        }

        this.profile.updatePreferences();
    }

    _buildExtension() {
        var _this2 = this;

        return _asyncToGenerator(function* () {
            var _context2;

            const zippedProfile = yield (0, _util.promisify)((_context2 = _this2.profile).encoded.bind(_context2))();

            if (Array.isArray(_this2.capabilities)) {
                _this2.capabilities.filter(function (capability) {
                    return capability.browserName === 'firefox';
                }).forEach(function (capability) {
                    _this2._setProfile(capability, zippedProfile);
                });

                return;
            }

            for (const browser in _this2.capabilities) {
                const capability = _this2.capabilities[browser].desiredCapabilities;

                if (!capability || capability.browserName !== 'firefox') {
                    continue;
                }

                _this2._setProfile(capability, zippedProfile);
            }
        })();
    }

    _setProfile(capability, zippedProfile) {
        // for older firefox and geckodriver versions
        capability.firefox_profile = zippedProfile;

        // for firefox >= 56.0 and geckodriver >= 0.19.0
        capability['moz:firefoxOptions'] = {
            profile: zippedProfile
        };
    }
}
exports.default = FirefoxProfileLauncher;
module.exports = exports['default'];